*********************************************************
* Sample size calculations
* Program:  xx
*
*  Authors: Brendon McConnell and Marcos Vera-Hernandez
*********************************************************

/* Notes:
   - In the paper "m" is used for cluster size, here "k" is used
   - Power = (1- Beta), but for program Power = Beta
   - All results are posted as rclass now
   - Other statistics are included (like success rate overall, Control and Treatment, etc)
   - No covariate case included within same program
   - Some control errors are included (e.g. theta must sum = 1)
   - New program included to call dialog
   */

cap prog drop power_binary
program power_binary, rclass
syntax anything [,alpha(real .05) beta(real .8) pi(real .5)]
tokenize "`0'",parse(" ,")

local Q          =`1'
local rho        =`2'
local k          =`3'

/* Q=0 ; No Covariates*/
if `Q'==0 {
/* syntax power_binary_covariate 0 rho k  p_0 impact */
local p_0     =`4'
local impact  =`5'
local t_alpha = invttail(1000,(`alpha'/2))
local t_beta  = -1*invttail(1000,`beta')
local deff    = (1+`rho'*(`k'-1))
local p_1     =`p_0'+`impact'

local p_T     = `p_1'
local p_C     = `p_0'

local n       =(`deff'/`k')*( (`p_1'*(1-`p_1')/`pi')+(`p_0'*(1-`p_0')/(1-`pi')) ) *((`t_alpha'+`t_beta')^2)/`impact'^2

di in gre _newline(2) "Note: No covariates assumed"
}



/* Q=2*/
if `Q'==2{
	local x_0        =`4'
	local theta_0    =`5'
	local p_C0       =`6'
	local impact_0   =`7'
	local x_1        =`8'
	local theta_1    =`9'
	local p_C1       =`10'
	local impact_1   =`11'
	local p_C        =(`theta_0'*`p_C0'+`theta_1'*`p_C1')
	local p_T0       =`p_C0'+`impact_0'
	local p_T1       =`p_C1'+`impact_1'
        /*newline*/ local p_T        =(`theta_0'*`p_T0'+`theta_1'*`p_T1')
	local deff       = (1+`rho'*(`k'-1))
	local impact     =(`theta_0'*`impact_0'+`theta_1'*`impact_1')

	#delimit ;

	matrix M=(
	(`pi'*`theta_0'*`p_T0'*(1-`p_T0') + (1-`pi')*`theta_0'*`p_C0'*(1-`p_C0')) + (`pi'*`theta_1'*`p_T1'*(1-`p_T1') + (1-`pi')*`theta_1'*`p_C1'*(1-`p_C1')) ,
	(`pi'*`theta_0'*`p_T0'*(1-`p_T0')) + (`pi'*`theta_1'*`p_T1'*(1-`p_T1')) ,
	`x_0'*(`pi'*`theta_0'*`p_T0'*(1-`p_T0') + (1-`pi')*`theta_0'*`p_C0'*(1-`p_C0')) + `x_1'*(`pi'*`theta_1'*`p_T1'*(1-`p_T1') + (1-`pi')*`theta_1'*`p_C1'*(1-`p_C1'))
	\
	(`pi'*`theta_0'*`p_T0'*(1-`p_T0')) + (`pi'*`theta_1'*`p_T1'*(1-`p_T1')) ,
	(`pi'*`theta_0'*`p_T0'*(1-`p_T0')) + (`pi'*`theta_1'*`p_T1'*(1-`p_T1')) ,
	`x_0'*(`pi'*`theta_0'*`p_T0'*(1-`p_T0')) + `x_1'*(`pi'*`theta_1'*`p_T1'*(1-`p_T1'))
	\
	`x_0'*(`pi'*`theta_0'*`p_T0'*(1-`p_T0') + (1-`pi')*`theta_0'*`p_C0'*(1-`p_C0')) + `x_1'*(`pi'*`theta_1'*`p_T1'*(1-`p_T1') + (1-`pi')*`theta_1'*`p_C1'*(1-`p_C1')) ,
	`x_0'*(`pi'*`theta_0'*`p_T0'*(1-`p_T0')) + `x_1'*(`pi'*`theta_1'*`p_T1'*(1-`p_T1')) ,
	`x_0'^2*(`pi'*`theta_0'*`p_T0'*(1-`p_T0') + (1-`pi')*`theta_0'*`p_C0'*(1-`p_C0')) + `x_1'^2*(`pi'*`theta_1'*`p_T1'*(1-`p_T1') + (1-`pi')*`theta_1'*`p_C1'*(1-`p_C1'))
	);
	mat invM=invsym(M);
	matrix g=(
	`theta_0'*(`p_T0'*(1-`p_T0') - `p_C0'*(1-`p_C0')) +  `theta_1'*(`p_T1'*(1-`p_T1') - `p_C1'*(1-`p_C1')),
	`theta_0'*(`p_T0'*(1-`p_T0')) +  `theta_1'*(`p_T1'*(1-`p_T1')),
	`theta_0'*`x_0'*(`p_T0'*(1-`p_T0') - `p_C0'*(1-`p_C0')) +  `theta_1'*`x_1'*(`p_T1'*(1-`p_T1') - `p_C1'*(1-`p_C1'))
	);
	matrix gprime=g';
	matrix A=g*invM*gprime;
	# delimit cr;
	local A=A[1,1]
        
        local aux = `theta_0' + `theta_1' 
        if `aux' !=1 {
                di in red "Error: Proportions (thetas) must add up to one"
                exit
                }
        } /* end Q=2*/

/* Q=3*/
else if `Q'==3{
	local x_0        =`4'
	local theta_0    =`5'
	local p_C0       =`6'
	local impact_0   =`7'
	local x_1        =`8'
	local theta_1    =`9'
	local p_C1       =`10'
	local impact_1   =`11'
	local x_2        =`12'
	local theta_2    =`13'
	local p_C2       =`14'
	local impact_2   =`15'
	local p_C        =(`theta_0'*`p_C0'+`theta_1'*`p_C1'+`theta_2'*`p_C2')
	local p_T0       =`p_C0'+`impact_0'
	local p_T1       =`p_C1'+`impact_1'
	local p_T2       =`p_C2'+`impact_2'
         /*newline*/ local p_T        =(`theta_0'*`p_T0'+`theta_1'*`p_T1'+`theta_2'*`p_T2') 
	local deff       = (1+`rho'*(`k'-1))
	local impact     =(`theta_0'*`impact_0'+`theta_1'*`impact_1'+`theta_2'*`impact_2')

	#delimit ;

	matrix M=(
	(`pi'*`theta_0'*`p_T0'*(1-`p_T0') + (1-`pi')*`theta_0'*`p_C0'*(1-`p_C0')) + 
	(`pi'*`theta_1'*`p_T1'*(1-`p_T1') + (1-`pi')*`theta_1'*`p_C1'*(1-`p_C1')) +
	(`pi'*`theta_2'*`p_T2'*(1-`p_T2') + (1-`pi')*`theta_2'*`p_C2'*(1-`p_C2')),
	
	(`pi'*`theta_0'*`p_T0'*(1-`p_T0')) + 
	(`pi'*`theta_1'*`p_T1'*(1-`p_T1')) + 
	(`pi'*`theta_2'*`p_T2'*(1-`p_T2')) ,
	
	`x_0'*(`pi'*`theta_0'*`p_T0'*(1-`p_T0') + (1-`pi')*`theta_0'*`p_C0'*(1-`p_C0')) + 
	`x_1'*(`pi'*`theta_1'*`p_T1'*(1-`p_T1') + (1-`pi')*`theta_1'*`p_C1'*(1-`p_C1')) + 
	`x_2'*(`pi'*`theta_2'*`p_T2'*(1-`p_T2') + (1-`pi')*`theta_2'*`p_C2'*(1-`p_C2'))
	\
	(`pi'*`theta_0'*`p_T0'*(1-`p_T0')) + 
	(`pi'*`theta_1'*`p_T1'*(1-`p_T1')) + 
	(`pi'*`theta_2'*`p_T2'*(1-`p_T2')) ,

	(`pi'*`theta_0'*`p_T0'*(1-`p_T0')) + 
	(`pi'*`theta_1'*`p_T1'*(1-`p_T1')) + 
	(`pi'*`theta_2'*`p_T2'*(1-`p_T2')) ,

	`x_0'*(`pi'*`theta_0'*`p_T0'*(1-`p_T0')) + 
	`x_1'*(`pi'*`theta_1'*`p_T1'*(1-`p_T1')) + 
	`x_2'*(`pi'*`theta_2'*`p_T2'*(1-`p_T2')) 
	\
	`x_0'*(`pi'*`theta_0'*`p_T0'*(1-`p_T0') + (1-`pi')*`theta_0'*`p_C0'*(1-`p_C0')) + 
	`x_1'*(`pi'*`theta_1'*`p_T1'*(1-`p_T1') + (1-`pi')*`theta_1'*`p_C1'*(1-`p_C1')) + 
	`x_2'*(`pi'*`theta_2'*`p_T2'*(1-`p_T2') + (1-`pi')*`theta_2'*`p_C2'*(1-`p_C2')) ,

	`x_0'*(`pi'*`theta_0'*`p_T0'*(1-`p_T0')) + 
	`x_1'*(`pi'*`theta_1'*`p_T1'*(1-`p_T1')) + 
	`x_2'*(`pi'*`theta_2'*`p_T2'*(1-`p_T2')) ,

	`x_0'^2*(`pi'*`theta_0'*`p_T0'*(1-`p_T0') + (1-`pi')*`theta_0'*`p_C0'*(1-`p_C0')) + 
	`x_1'^2*(`pi'*`theta_1'*`p_T1'*(1-`p_T1') + (1-`pi')*`theta_1'*`p_C1'*(1-`p_C1')) + 
	`x_2'^2*(`pi'*`theta_2'*`p_T2'*(1-`p_T2') + (1-`pi')*`theta_2'*`p_C2'*(1-`p_C2'))
	);
	mat invM=invsym(M);
	matrix g=(
	`theta_0'*(`p_T0'*(1-`p_T0') - `p_C0'*(1-`p_C0')) +  
	`theta_1'*(`p_T1'*(1-`p_T1') - `p_C1'*(1-`p_C1')) +  
	`theta_2'*(`p_T2'*(1-`p_T2') - `p_C2'*(1-`p_C2')),
	
	`theta_0'*(`p_T0'*(1-`p_T0')) +  
	`theta_1'*(`p_T1'*(1-`p_T1')) +  
	`theta_2'*(`p_T2'*(1-`p_T2')),
	
	`theta_0'*`x_0'*(`p_T0'*(1-`p_T0') - `p_C0'*(1-`p_C0')) +  
	`theta_1'*`x_1'*(`p_T1'*(1-`p_T1') - `p_C1'*(1-`p_C1')) +  
	`theta_2'*`x_2'*(`p_T2'*(1-`p_T2') - `p_C2'*(1-`p_C2')) 
	);
	matrix gprime=g';
	matrix A=g*invM*gprime;
	# delimit cr;
	local A=A[1,1]
                
        local aux = `theta_0' + `theta_1' + `theta_2'
        if `aux' !=1 {
                di in red "Error: Proportions (thetas) must add up to one"
                exit
                }
	} /* end Q=3*/		
	
	
/* Q=4*/
else if `Q'==4{
	local x_0        =`4'
	local theta_0    =`5'
	local p_C0       =`6'
	local impact_0   =`7'
	local x_1        =`8'
	local theta_1    =`9'
	local p_C1       =`10'
	local impact_1   =`11'
	local x_2        =`12'
	local theta_2    =`13'
	local p_C2       =`14'
	local impact_2   =`15'
	local x_3        =`16'
	local theta_3    =`17'
	local p_C3       =`18'
	local impact_3   =`19'
	
	local p_C        =(`theta_0'*`p_C0'+`theta_1'*`p_C1'+`theta_2'*`p_C2'+`theta_3'*`p_C3')
	local p_T0       =`p_C0'+`impact_0'
	local p_T1       =`p_C1'+`impact_1'
	local p_T2       =`p_C2'+`impact_2'
	local p_T3       =`p_C3'+`impact_3'
	 /*newline*/ local p_T        =(`theta_0'*`p_T0'+`theta_1'*`p_T1'+`theta_2'*`p_T2'+`theta_3'*`p_T3')	
        local deff       = (1+`rho'*(`k'-1))
	local impact     =(`theta_0'*`impact_0'+`theta_1'*`impact_1'+`theta_2'*`impact_2'+`theta_3'*`impact_3')

	#delimit ;

	matrix M=(
	(`pi'*`theta_0'*`p_T0'*(1-`p_T0') + (1-`pi')*`theta_0'*`p_C0'*(1-`p_C0')) + 
	(`pi'*`theta_1'*`p_T1'*(1-`p_T1') + (1-`pi')*`theta_1'*`p_C1'*(1-`p_C1')) +
	(`pi'*`theta_2'*`p_T2'*(1-`p_T2') + (1-`pi')*`theta_2'*`p_C2'*(1-`p_C2')) + 
	(`pi'*`theta_3'*`p_T3'*(1-`p_T3') + (1-`pi')*`theta_3'*`p_C3'*(1-`p_C3')),
	
	(`pi'*`theta_0'*`p_T0'*(1-`p_T0')) + 
	(`pi'*`theta_1'*`p_T1'*(1-`p_T1')) + 
	(`pi'*`theta_2'*`p_T2'*(1-`p_T2')) + 
	(`pi'*`theta_3'*`p_T3'*(1-`p_T3')) ,
	
	`x_0'*(`pi'*`theta_0'*`p_T0'*(1-`p_T0') + (1-`pi')*`theta_0'*`p_C0'*(1-`p_C0')) + 
	`x_1'*(`pi'*`theta_1'*`p_T1'*(1-`p_T1') + (1-`pi')*`theta_1'*`p_C1'*(1-`p_C1')) + 
	`x_2'*(`pi'*`theta_2'*`p_T2'*(1-`p_T2') + (1-`pi')*`theta_2'*`p_C2'*(1-`p_C2')) + 
	`x_3'*(`pi'*`theta_3'*`p_T3'*(1-`p_T3') + (1-`pi')*`theta_3'*`p_C3'*(1-`p_C3'))
	\
	(`pi'*`theta_0'*`p_T0'*(1-`p_T0')) + 
	(`pi'*`theta_1'*`p_T1'*(1-`p_T1')) + 
	(`pi'*`theta_2'*`p_T2'*(1-`p_T2')) + 
	(`pi'*`theta_3'*`p_T3'*(1-`p_T3')) ,

	(`pi'*`theta_0'*`p_T0'*(1-`p_T0')) + 
	(`pi'*`theta_1'*`p_T1'*(1-`p_T1')) + 
	(`pi'*`theta_2'*`p_T2'*(1-`p_T2')) + 
	(`pi'*`theta_3'*`p_T3'*(1-`p_T3')) ,

	`x_0'*(`pi'*`theta_0'*`p_T0'*(1-`p_T0')) + 
	`x_1'*(`pi'*`theta_1'*`p_T1'*(1-`p_T1')) + 
	`x_2'*(`pi'*`theta_2'*`p_T2'*(1-`p_T2')) + 
	`x_3'*(`pi'*`theta_3'*`p_T3'*(1-`p_T3')) 
	\
	`x_0'*(`pi'*`theta_0'*`p_T0'*(1-`p_T0') + (1-`pi')*`theta_0'*`p_C0'*(1-`p_C0')) + 
	`x_1'*(`pi'*`theta_1'*`p_T1'*(1-`p_T1') + (1-`pi')*`theta_1'*`p_C1'*(1-`p_C1')) + 
	`x_2'*(`pi'*`theta_2'*`p_T2'*(1-`p_T2') + (1-`pi')*`theta_2'*`p_C2'*(1-`p_C2')) + 
	`x_3'*(`pi'*`theta_3'*`p_T3'*(1-`p_T3') + (1-`pi')*`theta_3'*`p_C3'*(1-`p_C3')) ,

	`x_0'*(`pi'*`theta_0'*`p_T0'*(1-`p_T0')) + 
	`x_1'*(`pi'*`theta_1'*`p_T1'*(1-`p_T1')) + 
	`x_2'*(`pi'*`theta_2'*`p_T2'*(1-`p_T2')) + 
	`x_3'*(`pi'*`theta_3'*`p_T3'*(1-`p_T3')) ,

	`x_0'^2*(`pi'*`theta_0'*`p_T0'*(1-`p_T0') + (1-`pi')*`theta_0'*`p_C0'*(1-`p_C0')) + 
	`x_1'^2*(`pi'*`theta_1'*`p_T1'*(1-`p_T1') + (1-`pi')*`theta_1'*`p_C1'*(1-`p_C1')) + 
	`x_2'^2*(`pi'*`theta_2'*`p_T2'*(1-`p_T2') + (1-`pi')*`theta_2'*`p_C2'*(1-`p_C2')) + 
	`x_3'^2*(`pi'*`theta_3'*`p_T3'*(1-`p_T3') + (1-`pi')*`theta_3'*`p_C3'*(1-`p_C3'))
	);
	mat invM=invsym(M);
	matrix g=(
	`theta_0'*(`p_T0'*(1-`p_T0') - `p_C0'*(1-`p_C0')) +  
	`theta_1'*(`p_T1'*(1-`p_T1') - `p_C1'*(1-`p_C1')) +  
	`theta_2'*(`p_T2'*(1-`p_T2') - `p_C2'*(1-`p_C2')) +  
	`theta_3'*(`p_T3'*(1-`p_T3') - `p_C3'*(1-`p_C3')),
	
	`theta_0'*(`p_T0'*(1-`p_T0')) +  
	`theta_1'*(`p_T1'*(1-`p_T1')) +  
	`theta_2'*(`p_T2'*(1-`p_T2')) +  
	`theta_3'*(`p_T3'*(1-`p_T3')),
	
	`theta_0'*`x_0'*(`p_T0'*(1-`p_T0') - `p_C0'*(1-`p_C0')) +  
	`theta_1'*`x_1'*(`p_T1'*(1-`p_T1') - `p_C1'*(1-`p_C1')) +  
	`theta_2'*`x_2'*(`p_T2'*(1-`p_T2') - `p_C2'*(1-`p_C2')) +  
	`theta_3'*`x_3'*(`p_T3'*(1-`p_T3') - `p_C3'*(1-`p_C3')) 
	);
	matrix gprime=g';
	matrix A=g*invM*gprime;
	# delimit cr;
	local A=A[1,1]

        local aux = `theta_0' + `theta_1' + `theta_2' + `theta_3'
        if `aux' !=1 {
                di in red "Error: Proportions (thetas) must add up to one"
                exit
                }
        } /* end Q=4*/	
	
	
/* Q=5*/
else if `Q'==5{
	local x_0        =`4'
	local theta_0    =`5'
	local p_C0       =`6'
	local impact_0   =`7'
	local x_1        =`8'
	local theta_1    =`9'
	local p_C1       =`10'
	local impact_1   =`11'
	local x_2        =`12'
	local theta_2    =`13'
	local p_C2       =`14'
	local impact_2   =`15'
	local x_3        =`16'
	local theta_3    =`17'
	local p_C3       =`18'
	local impact_3   =`19'
	local x_4        =`20'
	local theta_4    =`21'
	local p_C4       =`22'
	local impact_4   =`23'

	local p_C        =(`theta_0'*`p_C0'+`theta_1'*`p_C1'+`theta_2'*`p_C2'+`theta_3'*`p_C3'+`theta_4'*`p_C4')
	local p_T0       =`p_C0'+`impact_0'
	local p_T1       =`p_C1'+`impact_1'
	local p_T2       =`p_C2'+`impact_2'
	local p_T3       =`p_C3'+`impact_3'
	local p_T4       =`p_C4'+`impact_4'
        /*newline*/ local p_T        =(`theta_0'*`p_T0'+`theta_1'*`p_T1'+`theta_2'*`p_T2'+`theta_3'*`p_T3'+`theta_4'*`p_T4')
	local deff       = (1+`rho'*(`k'-1))
	local impact     =(`theta_0'*`impact_0'+`theta_1'*`impact_1'+`theta_2'*`impact_2'+`theta_3'*`impact_3'+`theta_4'*`impact_4')

	#delimit ;

	matrix M=(
	(`pi'*`theta_0'*`p_T0'*(1-`p_T0') + (1-`pi')*`theta_0'*`p_C0'*(1-`p_C0')) + 
	(`pi'*`theta_1'*`p_T1'*(1-`p_T1') + (1-`pi')*`theta_1'*`p_C1'*(1-`p_C1')) +
	(`pi'*`theta_2'*`p_T2'*(1-`p_T2') + (1-`pi')*`theta_2'*`p_C2'*(1-`p_C2')) + 
	(`pi'*`theta_3'*`p_T3'*(1-`p_T3') + (1-`pi')*`theta_3'*`p_C3'*(1-`p_C3')) + 	
	(`pi'*`theta_4'*`p_T4'*(1-`p_T4') + (1-`pi')*`theta_4'*`p_C4'*(1-`p_C4')),
	
	(`pi'*`theta_0'*`p_T0'*(1-`p_T0')) + 
	(`pi'*`theta_1'*`p_T1'*(1-`p_T1')) + 
	(`pi'*`theta_2'*`p_T2'*(1-`p_T2')) + 
	(`pi'*`theta_3'*`p_T3'*(1-`p_T3')) + 
	(`pi'*`theta_4'*`p_T4'*(1-`p_T4')) ,
	
	`x_0'*(`pi'*`theta_0'*`p_T0'*(1-`p_T0') + (1-`pi')*`theta_0'*`p_C0'*(1-`p_C0')) + 
	`x_1'*(`pi'*`theta_1'*`p_T1'*(1-`p_T1') + (1-`pi')*`theta_1'*`p_C1'*(1-`p_C1')) + 
	`x_2'*(`pi'*`theta_2'*`p_T2'*(1-`p_T2') + (1-`pi')*`theta_2'*`p_C2'*(1-`p_C2')) + 
	`x_3'*(`pi'*`theta_3'*`p_T3'*(1-`p_T3') + (1-`pi')*`theta_3'*`p_C3'*(1-`p_C3')) + 
	`x_4'*(`pi'*`theta_4'*`p_T4'*(1-`p_T4') + (1-`pi')*`theta_4'*`p_C4'*(1-`p_C4'))
	\
	(`pi'*`theta_0'*`p_T0'*(1-`p_T0')) + 
	(`pi'*`theta_1'*`p_T1'*(1-`p_T1')) + 
	(`pi'*`theta_2'*`p_T2'*(1-`p_T2')) + 
	(`pi'*`theta_3'*`p_T3'*(1-`p_T3')) + 
	(`pi'*`theta_4'*`p_T4'*(1-`p_T4')) ,

	(`pi'*`theta_0'*`p_T0'*(1-`p_T0')) + 
	(`pi'*`theta_1'*`p_T1'*(1-`p_T1')) + 
	(`pi'*`theta_2'*`p_T2'*(1-`p_T2')) + 
	(`pi'*`theta_3'*`p_T3'*(1-`p_T3')) + 
	(`pi'*`theta_4'*`p_T4'*(1-`p_T4')) ,

	`x_0'*(`pi'*`theta_0'*`p_T0'*(1-`p_T0')) + 
	`x_1'*(`pi'*`theta_1'*`p_T1'*(1-`p_T1')) + 
	`x_2'*(`pi'*`theta_2'*`p_T2'*(1-`p_T2')) + 
	`x_3'*(`pi'*`theta_3'*`p_T3'*(1-`p_T3')) + 
	`x_4'*(`pi'*`theta_4'*`p_T4'*(1-`p_T4'))
	\
	`x_0'*(`pi'*`theta_0'*`p_T0'*(1-`p_T0') + (1-`pi')*`theta_0'*`p_C0'*(1-`p_C0')) + 
	`x_1'*(`pi'*`theta_1'*`p_T1'*(1-`p_T1') + (1-`pi')*`theta_1'*`p_C1'*(1-`p_C1')) + 
	`x_2'*(`pi'*`theta_2'*`p_T2'*(1-`p_T2') + (1-`pi')*`theta_2'*`p_C2'*(1-`p_C2')) + 
	`x_3'*(`pi'*`theta_3'*`p_T3'*(1-`p_T3') + (1-`pi')*`theta_3'*`p_C3'*(1-`p_C3')) + 
	`x_4'*(`pi'*`theta_4'*`p_T4'*(1-`p_T4') + (1-`pi')*`theta_4'*`p_C4'*(1-`p_C4')) ,

	`x_0'*(`pi'*`theta_0'*`p_T0'*(1-`p_T0')) + 
	`x_1'*(`pi'*`theta_1'*`p_T1'*(1-`p_T1')) + 
	`x_2'*(`pi'*`theta_2'*`p_T2'*(1-`p_T2')) + 
	`x_3'*(`pi'*`theta_3'*`p_T3'*(1-`p_T3')) + 
	`x_4'*(`pi'*`theta_4'*`p_T4'*(1-`p_T4')) ,

	`x_0'^2*(`pi'*`theta_0'*`p_T0'*(1-`p_T0') + (1-`pi')*`theta_0'*`p_C0'*(1-`p_C0')) + 
	`x_1'^2*(`pi'*`theta_1'*`p_T1'*(1-`p_T1') + (1-`pi')*`theta_1'*`p_C1'*(1-`p_C1')) + 
	`x_2'^2*(`pi'*`theta_2'*`p_T2'*(1-`p_T2') + (1-`pi')*`theta_2'*`p_C2'*(1-`p_C2')) + 
	`x_3'^2*(`pi'*`theta_3'*`p_T3'*(1-`p_T3') + (1-`pi')*`theta_3'*`p_C3'*(1-`p_C3')) + 
	`x_4'^2*(`pi'*`theta_4'*`p_T4'*(1-`p_T4') + (1-`pi')*`theta_4'*`p_C4'*(1-`p_C4'))
	);
	mat invM=invsym(M);
	matrix g=(
	`theta_0'*(`p_T0'*(1-`p_T0') - `p_C0'*(1-`p_C0')) +  
	`theta_1'*(`p_T1'*(1-`p_T1') - `p_C1'*(1-`p_C1')) +  
	`theta_2'*(`p_T2'*(1-`p_T2') - `p_C2'*(1-`p_C2')) +  
	`theta_3'*(`p_T3'*(1-`p_T3') - `p_C3'*(1-`p_C3')) +  
	`theta_4'*(`p_T4'*(1-`p_T4') - `p_C4'*(1-`p_C4')),
	
	`theta_0'*(`p_T0'*(1-`p_T0')) +  
	`theta_1'*(`p_T1'*(1-`p_T1')) +  
	`theta_2'*(`p_T2'*(1-`p_T2')) +  
	`theta_3'*(`p_T3'*(1-`p_T3')) +  
	`theta_4'*(`p_T4'*(1-`p_T4')),
	
	`theta_0'*`x_0'*(`p_T0'*(1-`p_T0') - `p_C0'*(1-`p_C0')) +  
	`theta_1'*`x_1'*(`p_T1'*(1-`p_T1') - `p_C1'*(1-`p_C1')) +  
	`theta_2'*`x_2'*(`p_T2'*(1-`p_T2') - `p_C2'*(1-`p_C2')) +  
	`theta_3'*`x_3'*(`p_T3'*(1-`p_T3') - `p_C3'*(1-`p_C3')) +  
	`theta_4'*`x_4'*(`p_T4'*(1-`p_T4') - `p_C4'*(1-`p_C4'))
	);
	matrix gprime=g';
	matrix A=g*invM*gprime;
	# delimit cr;
	local A=A[1,1]

        local aux = `theta_0' + `theta_1' + `theta_2' + `theta_3' + `theta_4'
        if `aux' !=1 {
                di in red "Error: Proportions (thetas) must add up to one"
                exit
                }
	} /* end Q=5*/
	
	
	
if `Q' !=0 local n = (`deff'/`k')*(`A') *((invttail(1000,(1-`beta'))+invttail(1000,`alpha'/2))^2)/`impact'^2


* Check parameter values
  if `rho'>1 | `rho'<-1 {
     di in red "Error: ICC must be between -1 and 1" 
     exit()
     }
  
  if `alpha'>1 | `alpha'<0 {
     di in red "Error: significance must be between 0 and 1" 
     exit()
     }
 
  if `beta'>1 | `beta'<0 {
     di in red "Error: beta must be between 0 and 1" 
     exit()
     }




* DISPLAY AND POST RESULTS *************************
if `rho' != 0 {
        loc rho_yesno "yes"
        return scalar N_clusters = `n' 
        loc nobs = `n'*`k'
        return scalar N_obs = `nobs'
        }


if `rho' == 0 {
      loc rho_yesno "no"
      return scalar N_clusters = `n' 
      loc nobs = `n'
      return scalar N_obs = `nobs'
      loc k = 1
      }


return scalar N_T_cl = `n'*`pi'
return scalar N_C_cl = `n'*(1-`pi')
return scalar Nobs_T_cl = `nobs'*`pi'
return scalar Nobs_C_cl = `nobs'*(1-`pi')
return scalar Ovl_impact = `impact'
return scalar Ovl_scss_T = `p_T' 
return scalar Ovl_scss_C = `p_C' 


* Display
di in smcl in gr _newline(2) "{hline 75}"
di in gr  _col(3) "Sample size calculations"
di in smcl in gr "{hline 75}"
di in gr _col(3) "Input Parameters:"
	
	di in gr _col(7) "Covariate takes " in ye "`Q'" in gr " values"
	di in gr _col(7) "Covariate values: "  _col(45) "`x_0' `x_1' `x_2' `x_3' `x_4'"
	di in gr _col(7) "Proportions (thetas): "  _col(45) "`theta_0' `theta_1' `theta_2' `theta_3' `theta_4'"
	di in gr _col(7) "Effect Size: "  _col(45) "`impact_0' `impact_1' `impact_2' `impact_3' `impact_4'"
	di in gr _col(7) "Control Group Rates: "  _col(45) "`p_C0' `p_C1' `p_C2' `p_C3' `p_C4'"
	di in gr _col(7) "Cluster design: "  _col(45) "`rho_yesno'"
	di in gr _col(7) "Cluster size: "  _col(45) "`k'"
	di in gr _col(7) "Intra Cluster Correlation: "  _col(45) "`rho'"
	di in gr _col(7) "Significance level: "  _col(45) "`alpha'"
	di in gr _col(7) "Power level: "  _col(45) "`beta'"
	di in gr _col(7) "Treatment Gr sampling rate (Pi): "  _col(45) "`pi'"
di in smcl in gr "{hline 75}"

di in gr _col(3) "Sample Size: "
        di in gr _col(7) "Number of Clusters = " in ye _col(45) round(`n',0.1)
        di in gr _col(7) "Number of Observations = " in ye _col(45) round(`nobs',0.1)
        di in gr _col(7) "Number of Treatment Clusters = " in ye _col(45) round(`n'*`pi',0.1)
        di in gr _col(7) "Number of Control Clusters = " in ye _col(45) round(`n'*(1-`pi'),0.1)
        di in gr _col(7) "Number of Treatment Observations = " in ye _col(45) round(`nobs'*`pi',0.1)
        di in gr _col(7) "Number of Control Observations = " in ye _col(45) round(`nobs'*(1-`pi'),0.1)
        di in gr _col(7) "Overall Impact = " in ye _col(45) round(`impact',0.001)
        di in gr _col(7) "Overall Success Rate for Treatment= " in ye _col(45) round(`p_T',0.01)
        di in gr _col(7) "Overall Success Rate for Controls= " in ye _col(45) round(`p_C',0.01)

di in smcl in gr "{hline 75}"

end



**************************
* Program to call dialog *
**************************
cap prog drop power_binary_dialog
program power_binary_dialog, rclass
db dialog_binary
end


